<?php
namespace dsmirror;
require_once 'dsmirror/RESTClient.php';

/**
 *
 * @author Tecknolab
 *        
 */
class RESTClientWP extends RESTClient implements IRESTClient
{
    private $args;
    
    /**
     */
    public function __construct($endpoint, $appId, $secret, $appVersion)
    {
        parent::__construct($endpoint, $appId, $secret, $appVersion, 'WP');
    }
    
    function __destruct() {
    }
    
    public function doCall($context, $outFormat="")
    {
        if (!isset($this->args)) {
            throw new \Exception("Invalid REST doCall with no previous setHttpParams");
        }
        if (isset($this->appId) && isset($this->secret))
        {
            $this->args['headers']['Authorization'] = 'Basic ' . base64_encode($this->appId.':'.$this->getAuthToken());
        }
        try {
            $response = \wp_remote_request( $this->args['path'], $this->args );
        } finally {
            unset($this->args);
        }
        if ($outFormat === "NONE") {
            if ($this->isError($response, 204)) {
                $this->processError($response, $context);
            }
            return;
        }
        
        if ($this->isError($response)) {
            $this->processError($response, $context);
        }
        trigger_error("REST-WP response: ".\wp_remote_retrieve_body($response), E_USER_NOTICE);
        if ($outFormat === "STRING")
        {
            return \wp_remote_retrieve_body($response);
        }
        $res = json_decode(\wp_remote_retrieve_body($response));
        return $res;
    }
    
    public function setHttpParams($path, $method="GET", $obj=NULL, $outFormat="")
    {
        $this->args = array();
        $this->args['method'] = $method;
        $this->args['path'] = $this->endpoint . $path;
        if ($outFormat == "STRING") {
            $this->args['headers'] = array(
                "Accept" => "text/plain",
                "Content-Type" => "application/json"
            );
        }
        else {
            $this->args['headers'] = array(
                 "Accept" => "application/json",
                 "Content-Type" => "application/json"
            );
        }
        foreach ($this->commonHeaders as $header => $headerValue) {
            if (!empty($headerValue)) {$this->args['headers'][$header] = $headerValue;}
        }
        
        if (($method == "POST" || $method == "PUT") && $obj != NULL) {
            $data = json_encode($obj);
            trigger_error("REST-WP client path ".$this->endpoint . $path. " " . $method. " with payload ".$data. " headers ".json_encode($this->args['headers']) , E_USER_NOTICE);
            $this->args['body'] = $data;
        }
        else {
            trigger_error("REST-WP client path ".$this->endpoint . $path. " ".$method ." headers ".json_encode($this->args['headers']) , E_USER_NOTICE);
        }
    }
    
    protected function isError($res, $okStatus = 200)
    {
        return (wp_remote_retrieve_response_code( $res ) != $okStatus);
    }
    
    protected function processError($res, $context)
    {
        trigger_error("Processing error from response ".json_encode($res) , E_USER_NOTICE);
        if ( is_wp_error( $res ) ) {
            $error_code = array_key_first( $res->errors );
            $error_message = $res->errors[$error_code][0];
            throw new RESTException("Error ". $context . ". Status code: " . $error_code . " Error: " . (!empty($error_message) ? $error_message : "<not specified>"), 0);
        }
        if ($res != NULL) {
            throw new RESTException("Error ". $context . ". Status code: " . wp_remote_retrieve_response_code( $res ) . " Error: " . (isset($res['body'])?$res['body']:(isset($res['response']['message'])?$res['response']['message']:"<not specified>")), wp_remote_retrieve_response_code( $res ));
        } else {
            throw new \Exception("Error ". $context . ". No response.");
        }
    }
    
}
