<?php
/**
 * Plugin Name: DSmirror data replication
 * Plugin URI: https://dsmirror.tecknolab.com/
 * Description: DSmirror (datasource mirror) data replication tool for wordpress.
 * Version: 1.01.04
 * Author: Tecknolab
 * Author URI: https://tecknolab.com/
 * License: GPLv2
 **/
require_once 'dsmirror/dsmirror-plugin-back.php';
require_once 'dsmirror/dsmirror-plugin-front.php';
require_once 'RESTClientWP.php';

const REPLICA_EXECUTION_SCHED_HOOK='dsmirror_replica_execution';

add_action('admin_menu', 'dsmirror_plugin_setup_menu');

function dsmirror_cron_schedules($schedules){
    if(!isset($schedules["dsmirror"])){
        $schedules["dsmirror"] = array(
            'interval' => 60,
            'display' => __('Once every minute'));
    }
    return $schedules;
}
add_filter('cron_schedules','dsmirror_cron_schedules');

function dsmirror_plugin_setup_menu(){
    $page_hook_suffix = add_submenu_page('tools.php', 'DSmirror settings', 'DSmirror', 'manage_options', 'dsmirror-plugin', 'dsmirror_admin_home' );
    /*
     * Use the retrieved $page_hook_suffix to hook the function that links our script.
     * This hook invokes the function only on our plugin administration screen,
     * see: https://developer.wordpress.org/reference/hooks/admin_print_scripts-hook_suffix/
     */
    add_action( "admin_print_scripts-{$page_hook_suffix}", 'dsmirror_enqueue_scripts');
}

function dsmirror_activate() {
    if ( ! wp_next_scheduled( REPLICA_EXECUTION_SCHED_HOOK ) ) {
        wp_schedule_event( time(), 'dsmirror', REPLICA_EXECUTION_SCHED_HOOK );
        trigger_error("Activated cron for dsmirror. Next run: ".gmdate("Y-m-d\TH:i:s\Z",wp_next_scheduled( REPLICA_EXECUTION_SCHED_HOOK )), E_USER_NOTICE); 
    }
}
register_activation_hook( __FILE__, 'dsmirror_activate' );

function dsmirror_deactivate() {
    $timestamp = wp_next_scheduled( REPLICA_EXECUTION_SCHED_HOOK );
    if ($timestamp){
        wp_unschedule_event( $timestamp, REPLICA_EXECUTION_SCHED_HOOK );
        trigger_error("Removed cron for dsmirror on plugin deactivation", E_USER_NOTICE);
    }
}

function dsmirror_next_scheduled() {
    return wp_next_scheduled( REPLICA_EXECUTION_SCHED_HOOK );
}

function dsmirror_schedule_engine() {
    return 'WP_cron';
}

function dsmirror_schedule_engine_info() {    
    return DSMIRROR_URL."/wp_cron";
}

function dsmirror_enqueue_scripts() {
    wp_enqueue_script( 'jquery' );
}
function dsmirror_jquery_script() {
    return '';
}

register_deactivation_hook( __FILE__, 'dsmirror_deactivate' );

function dsmirror_uninstall() {
    delete_option(CONFIG_PARAM_REGISTRATION);
    delete_option(CONFIG_PARAM_REPLICAS);
    delete_option(CONFIG_PARAM_EXECUTION);
    trigger_error("Removed dsmirror configuration settings", E_USER_NOTICE);
}

register_uninstall_hook( __FILE__, 'dsmirror_uninstall' );

add_action( REPLICA_EXECUTION_SCHED_HOOK, 'dsmirror_replica_sched' );

// This function creates the output for the admin page.
// General check for user permissions.
function dsmirror_admin_home(){
    if (!current_user_can('manage_options'))  {
        wp_die( __('You do not have sufficient priviledges to access this page.')    );
    }
    dsmirror_home(function($appId,  $secret){return new dsmirror\RESTClientWP(DSMIRROR_ENDPOINT, $appId,  $secret, DSMIRROR_VERSION);});
}

function dsmirror_replica_sched() {
    dsmirror_replica_exec(function($appId,  $secret){return new dsmirror\RESTClientWP(DSMIRROR_ENDPOINT, $appId,  $secret, DSMIRROR_VERSION);});
}
