<?php
namespace dsmirror;

/**
 *
 * @author Tecknolab
 *        
 */
class DSConfig
{
    public function __construct($obj)
    {
        if ($obj != NULL) {
            if ($obj->id == NULL || $obj->id == "") {
                throw new \Exception("Datasource config id cannot be empty or null");
            }
            $this->id = $obj->id;
            $this->tableName = $obj->tableName;
            $this->pk = $obj->pk;
            $this->attributes = $obj->attributes;
            $this->reverse = $obj->reverse;
        }
    }
    
    public $id;
    public $tableName;
    public /*Array*/ $pk;
    public /*Array*/ $attributes;
    public $reverse = false;
}

class DSReplicaOptions {
    public function __construct($obj)
    {
        if ($obj != NULL) {
            $this->targetDBUserAgent = $obj->targetDBUserAgent;
            $this->targetTableName = $obj->targetTableName;
            $this->createTable = $obj->createTable;
            $this->test = $obj->test;
        }
    }
    public $targetDBUserAgent;
    public $targetTableName;
    public /*boolean*/ $createTable = false;
    public /*boolean*/ $test = false;
}

class DSReplica
{
    public function __construct($obj)
    {
        if ($obj != NULL) {
            $this->dsConfigId = $obj->dsConfigId;
            $this->dsReplicaId = $obj->dsReplicaId;
            if (isset($obj->options))
                $this->options = new DSReplicaOptions($obj->options);
        }
    }
    
    public $dsConfigId;
    public $dsReplicaId;
    public /*DSReplicaOptions*/ $options;
}

class ReplicaExec
{
    public function __construct($obj)
    {
        if ($obj != NULL) {
            if (isset($obj->dsReplica)) {
               $this->dsReplica = new DSReplica($obj->dsReplica);
            }
            if (isset($obj->dsConfig)) {
                $this->dsConfig = new DSConfig($obj->dsConfig);
            }
            $this->timeout = $obj->timeout;
            $this->timedout = $obj->timedout;
            $this->jobId = $obj->jobId;
        } else {
            $this->timedout=false;
        }
    }
    
    public /*DSReplica*/ $dsReplica;
    public /*DSConfig*/ $dsConfig;
    public /* int */ $timeout;
    public /*boolean*/ $timedout;
    public $jobId; /*inprocess jobid in timedout jobs*/
}

class ReplicationContext
{
    public function __construct($obj)
    {
        if ($obj != NULL) {
            if ($obj->pending != NULL) {
                $this->pending = new ReplicationMap($obj->pending);
            }
            if ($obj->lastPartialSegment != NULL) {
                $this->lastPartialSegment = new Segment($obj->lastPartialSegment);
            }
            $this->nextPartialFilter = $obj->nextPartialFilter;
        }
    }
    public /*ReplicationMap*/ $pending;
    public /*Segment*/ $lastPartialSegment;
    public $nextPartialFilter;
}

class Segment {
    public function __construct($obj)
    {
        if ($obj != NULL) {
            if ($obj->size < 0) {
                throw new \Exception("segment size must be >= 0: " . json_encode($obj));
            }
            $this->pkIniExcluded = $obj->pkIniExcluded;
            $this->pkEndIncluded = $obj->pkEndIncluded;
            $this->size = $obj->size;
        }
    }
    public /*Array*/ $pkIniExcluded;
    public /*Array*/ $pkEndIncluded;
    public $size = 0;
}
class SegmentHashDescriptor {
    public function __construct($obj)
    {
        if ($obj != NULL) {
            if ($obj->segment == NULL) {
                throw new \Exception("segment cannot be empty or null");
            }
            $this->segment = new Segment($obj->segment);
            $this->segmentFilter = $obj->segmentFilter;
        }
    }
    public /*Segment*/ $segment;
    public $segmentFilter;
}
class SegmentHash {
    public /*Segment*/ $segment;
    public $hash;
}
class ContentHashItem {
    public /*Array*/ $row;
    public $hash;
}
class SegmentContentHashResult {
    public /*Segment*/ $segment;
    public /*Array*/ $rowHashList; /* Array of ContentHashItem */
    public $hashClient;
    public /*Segment*/ $partialSegment;
}
class ReplicationMap
{
    public function __construct($obj)
    {
        if ($obj != NULL) {
            if (empty($obj->jobId)) {
                throw new \Exception("Replication job id cannot be empty or null");
            }
            $this->jobId = $obj->jobId;
            $this->dsReplicaId = $obj->dsReplicaId;
            $this->maxContentRowsPerRequest = $obj->maxContentRowsPerRequest;
            $this->selectQuery = $obj->selectQuery;
            $this->segmentHashQuery = $obj->segmentHashQuery;
            $this->insertStmt = $obj->insertStmt;
            $this->deleteStmt = $obj->deleteStmt;
            $this->updateStmt = $obj->updateStmt;
            $this->orderBy = $obj->orderBy;
            for ($x=0; $x < count($obj->segmentsList); ++$x) {
                $this->segmentsList[]= new SegmentHashDescriptor($obj->segmentsList[$x]);
            }
        }
    }
    public $jobId;
    public $dsReplicaId;
    public /*Array*/ $segmentsList; /* Array of SegmentHashDescriptor */
    public /*int*/ $maxContentRowsPerRequest;
    public $selectQuery;
    public $segmentHashQuery;
    public $insertStmt;
    public $deleteStmt;
    public $updateStmt;
    public $orderBy;
    
}
class Command {
    public $operation;
    public $target;
}
class ReplicationDelta {
    public function __construct($obj)
    {
        if ($obj != NULL) {
            if ($obj->commandList != NULL) {
                $this->commandList = $obj->commandList;
            }
            if ($obj->partialSegment != NULL) {
                $this->partialSegment = new Segment($obj->partialSegment);
            }
            $this->nextPartialFilter = $obj->nextPartialFilter;
        }
        else {
            $this->commandList = array();
        }
    }
    public /*Array*/ $commandList; /* Array of Command */
    public /*Segment*/ $partialSegment;
    public $nextPartialFilter;
}

class ReplicationDeltaResultItem {
    public $operation;
    public /*Array*/ $row;
}

class ReplicationDeltaResult {
    public /*SegmentHash*/ $segmentHash;
    public /*Array(ReplicationDeltaResultItem)*/ $operationResult;
    public /*Segment*/ $partialSegment;
}

class ReplicationError {
    public /*Segment*/ $segment;
    public $error=200; // should match http status convention
    public $errorDescription;
}
