<?php
namespace dsmirror;
require_once 'IReplicationService.php';
require_once 'RESTClient.php';
require_once 'Model.php';

/**
 *
 * @author Tecknolab
 *        
 */
final class ReplicationServiceClient implements IReplicationService
{
    private $irestclient; /* IRESTClient */
    private function __construct()
    {
        
    }
    function __destruct() {
        unset($irestclient);
    }
    
    public static function createCustom ($irestclient)/*: ReplicationServiceClient*/
    {
        $res = new ReplicationServiceClient();
        $res->irestclient = $irestclient;
        return $res;
    }
    
    
    public function properties() /* : Array of properties */
    {
        $this->irestclient->setHttpParams("/api");
        return $this->irestclient->doCall("retrieving DSmirror agent properties");
    }

    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::getDSList()
     */
    public function getDSList() /*: iterable*/
    {
        $this->irestclient->setHttpParams(ReplicationServiceClient::DSLIST_PATH);
        return $this->irestclient->doCall("retrieving datasource list");
    }

    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::getDSListReverse()
     */
    public function getDSListReverse() /*: iterable*/
    {
        $this->irestclient->setHttpParams(ReplicationServiceClient::DSLIST_REVERSE_PATH);
        return $this->irestclient->doCall("retrieving table list in outbound replication");
    }
    
    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::generateCandidateReverseDSConfigScript()
     */
    public function generateCandidateReverseDSConfigScript ($targetDB) /*: string*/
    {
        if ($targetDB != NULL) {
            $params = "?target=".$targetDB;
        } else {
            $params = "";
        }
        $this->irestclient->setHttpParams(ReplicationServiceClient::SCRIPT_REVERSE_CANDIDATE_PATH . $params, "GET", NULL, "STRING");
        return $this->irestclient->doCall("retrieving outbound replication candidate tables script", "STRING");
    }
    
    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::registerCandidateReverseDSConfig()
     */
    public function registerCandidateReverseDSConfig ($metadata)/*: iterable*/ //DSConfig iterable
    {
        $this->irestclient->setHttpParams(ReplicationServiceClient::REGISTER_REVERSE_CANDIDATE_PATH, "POST", $metadata);
        return $this->irestclient->doCall("registering candidate tables for outbound replication");       
    }
    
    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::registerDSReplica()
     */
    public function registerDSReplica($dsConfigId, $dsReplicaOptions) /*: DSReplica*/
    {
        $this->irestclient->setHttpParams(ReplicationServiceClient::DSREPLICA_PATH."/".$dsConfigId, "POST", $dsReplicaOptions);
        return new DSReplica($this->irestclient->doCall("creating table replica"));        
    }

    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::generateTargetDSScript()
     */
    public function generateTargetDSScript($dsReplicaId) /*: string*/
    {
        $this->irestclient->setHttpParams(ReplicationServiceClient::SCRIPT_PATH."/".$dsReplicaId, "GET", NULL, "STRING");
        return $this->irestclient->doCall("generating replica table creation script", "STRING");
    }
    
    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::deleteReplica()
     */
    public function deleteReplica($dsReplicaId)
    {
        $this->irestclient->setHttpParams(ReplicationServiceClient::DELETE_REPLICA_PATH."/".$dsReplicaId, "DELETE");
        $this->irestclient->doCall("deleting replica", "NONE");
    }

    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::startSyncDS()
     */
    public function startSyncDS ($dsReplicaId) /*: ReplicationMap*/
    {
        $this->irestclient->setHttpParams(ReplicationServiceClient::START_REPLICA_PATH."/".$dsReplicaId, "POST", NULL);
        return new ReplicationMap($this->irestclient->doCall("starting new replica"));        
    }

    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::getReplicationDirtySegments()
     */
    public function getReplicationDirtySegments ($jobId, $segmentHashArray) /*: iterable*/
    {
        $this->irestclient->setHttpParams(ReplicationServiceClient::DIRTY_SEGMENTS_PATH."/".$jobId, "POST", $segmentHashArray);
        $outputArray =  $this->irestclient->doCall("getting replica dirty segments");
        $res = array();
        for($x = 0; $x < count($outputArray); $x++) {
            $res[$x] = new SegmentHashDescriptor($outputArray[$x]);
        }
        return $res;        
    }

    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::getSegmentReplicationDelta()
     */
    public function getSegmentReplicationDelta ($jobId, $segmentContentHashResult) /*: ReplicationDelta*/
    {
        $this->irestclient->setHttpParams(ReplicationServiceClient::REPLICATION_DELTA_PATH."/".$jobId, "POST", $segmentContentHashResult);
        return new ReplicationDelta($this->irestclient->doCall("getting replication delta"));
    }
    
    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::confirmReplicationDelta()
     */
    public function confirmReplicationDelta ($jobId, $replicationDeltaResult)
    {
        $this->irestclient->setHttpParams(ReplicationServiceClient::CONFIRM_DELTA_PATH."/".$jobId, "PUT", $replicationDeltaResult);
        $this->irestclient->doCall("confirming replication delta execution", "NONE");
    }
    
    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::abortReplicationDelta()
     */
    public function abortReplicationDelta ($jobId, $replicationError)
    {
        $this->irestclient->setHttpParams(ReplicationServiceClient::ABORT_DELTA_PATH."/".$jobId, "PUT", $replicationError);
        $this->irestclient->doCall("aborting replication delta execution", "NONE");
    }
    
    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::finishSyncDS()
     */
    public function finishSyncDS($jobId)
    {
        $this->irestclient->setHttpParams(ReplicationServiceClient::FINISH_REPLICA_PATH."/".$jobId, "DELETE");
        $this->irestclient->doCall("deleting replica job", "NONE");
    }
    
    /**
     * (non-PHPdoc)
     *
     * @see \dsmirror\IReplicationService::getReplicationContext()
     */
    public function getReplicationContext($jobId) /* ReplicationContext */
    {
        $this->irestclient->setHttpParams(ReplicationServiceClient::CONTEXT_PATH."/".$jobId);
        return new ReplicationContext($this->irestclient->doCall("retrieving job context"));
    }
    
    const DSLIST_PATH = "/api/ds";
    const DSLIST_REVERSE_PATH = "/api/ds/reverse";
    const REGISTER_REVERSE_CANDIDATE_PATH = "/api/ds/reverse/register";
    const SCRIPT_REVERSE_CANDIDATE_PATH = "/api/script/reverse";
    const DSREPLICA_PATH = "/api/replica";
    const SCRIPT_PATH = "/api/script/replica";
    const DELETE_REPLICA_PATH = "/api/delete";
    const START_REPLICA_PATH = "/api/start";
    const DIRTY_SEGMENTS_PATH = "/api/dirty";
    const REPLICATION_DELTA_PATH = "/api/delta";
    const CONFIRM_DELTA_PATH = "/api/confirm";
    const ABORT_DELTA_PATH = "/api/abort";
    const FINISH_REPLICA_PATH = "/api/finish";
    const CONTEXT_PATH = "/api/context";
}

