<?php
/**
 * Author: Tecknolab
 * Author URI: https://tecknolab.com/
 * License: GPLv2
 **/
require_once 'dsmirror-plugin-common.php';


function dsmirror_replica_exec($restClientSetup) {
    $initRun = time();
    trigger_error("Activated cron execution for dsmirror!", E_USER_NOTICE);
    $registration = get_option(CONFIG_PARAM_REGISTRATION, Array(REGISTRATION_ACTIVE=>false, REGISTRATION_REVERSE=>false));
    $registered = $registration[REGISTRATION_ACTIVE];
    $execution = get_option(CONFIG_PARAM_EXECUTION, Array(EXECUTION_STATUS=>EXEC_NEVER_STATUS, EXECUTION_STATS=>'',EXECUTION_INDEX=>0,EXECUTION_START_TS=>NULL,EXECUTION_END_TS=>NULL,EXECUTION_REPLICAS=>Array()) );
    $replicas = get_option(CONFIG_PARAM_REPLICAS, array(REPLICAS_LIST=>null,REPLICAS_CREATE_TABLE=>true,REPLICAS_SCHEDULING=>SCHED_NEVER));
    if ($registered)
    {
        $currentReplicaIndex = $execution[EXECUTION_INDEX];
        $dsReplicaArray = $replicas[REPLICAS_LIST];
        if (!empty($replicas[REPLICAS_LIST]) && $currentReplicaIndex >= count($replicas[REPLICAS_LIST]))
        {
            trigger_error("Current Replica Index ".$currentReplicaIndex." greater or equal than num of replicas ".count($replicas[REPLICAS_LIST]).". Initializing back to 0" , E_USER_NOTICE);
            $currentReplicaIndex =  0;
        }
        if ($execution[EXECUTION_STATUS]==EXEC_RUNNING_STATUS && (time() - $execution[EXECUTION_LAST_TS]) < 2*DEFAULT_TIMEOUT) {
            trigger_error("Replication in RUNNING status. Ignore cron launch" , E_USER_NOTICE);
            return;
        }
        if ($replicas[REPLICAS_SCHEDULING] === SCHED_NEVER)
        {
            trigger_error("Replication scheduling disabled. Ignoring run ", E_USER_NOTICE);
            return;
        }
        if ($currentReplicaIndex == 0 && !empty($dsReplicaArray) && !$dsReplicaArray[0]['replica']->timedout) {
            // Check scheduling
            if (isset($execution[EXECUTION_START_TS]) && $currentReplicaIndex == 0 && 
                time()<($execution[EXECUTION_START_TS]+SCHED_SECONDS_ARRAY[$replicas[REPLICAS_SCHEDULING]]))
            {
                trigger_error("Replication scheduling not yet reached. Time to run ". (($execution[EXECUTION_START_TS]+SCHED_SECONDS_ARRAY[$replicas[REPLICAS_SCHEDULING]]) - time())." seconds", E_USER_NOTICE);            
                return;
            }
            $execution[EXECUTION_START_TS]=time();
        }
        try {            
            $connection = mysqli_connect(DB_HOST, DB_USER, DB_PASSWORD);
            mysqli_select_db($connection, DB_NAME);
            trigger_error("Connected to DB. Starting replicas", E_USER_NOTICE);
            $regReverse = $registration[REGISTRATION_REVERSE];
            trigger_error("Current registration reverse ". $regReverse, E_USER_NOTICE);
            $execCode=EXEC_RUNNING_STATUS;
            $execution[EXECUTION_STATUS]=$execCode;
            $execution[EXECUTION_LAST_TS]=time();
            $numActiveTables = dsmirror_num_active($dsReplicaArray);
            if ($regReverse) {
                $numActiveTables += dsmirror_num_active($dsReplicaArray, $regReverse);
            }
            $execution[EXECUTION_STATS]="Processing ".$numActiveTables.($numActiveTables == 1) ? " table." : " tables.";
            update_option(CONFIG_PARAM_EXECUTION, $execution);
            $appId = $registration[REGISTRATION_APPID];
            $appSecret = $registration[REGISTRATION_SECRET];
            $createTable = $replicas[REPLICAS_CREATE_TABLE];
            $restClient = $restClientSetup($appId,$appSecret);
            $replicationManager = new dsmirror\ReplicationManager(dsmirror\ReplicationServiceClient::createCustom($restClient));
            $processedTables = 0;
            $failedTables = array();
            $stats = "";
            for ($index = $currentReplicaIndex; $index < count($dsReplicaArray); ++$index) {
                $replica = $dsReplicaArray[$index];
                trigger_error("Processing for execution replica ".json_encode($replica), E_USER_NOTICE);
                // Should not happen but if is reverse replica and is not active, skip it
                if ($replica['reverse'] && !$regReverse) {
                    trigger_error("Ignoring outbound replica ".$replica['source']. " since outbound replication is disabled", E_USER_NOTICE);
                    continue;
                }
                if ($replica['found'] && $replica['active']) {
                    if (isset($execution[EXECUTION_REPLICAS][$replica['ds']->id])) {
                        $replicaExec = $execution[EXECUTION_REPLICAS][$replica['ds']->id];
                    } else {
                        $replicaExec = new dsmirror\ReplicaExec(NULL);
                        $replicaExec->dsConfig = $replica['ds'];
                    }
                    if (!isset($replicaExec->dsReplica)) {
                        $replicaExec->dsReplica = new dsmirror\DSReplica(NULL);
                    }
                    if (!isset($replicaExec->dsReplica->options)) {
                        $replicaExec->dsReplica->options = new dsmirror\DSReplicaOptions(NULL);
                    }
                    $replicaExec->dsReplica->options->createTable=$createTable;
                    $replicaExec->dsReplica->options->targetTableName=$replica['target'];
                    $replicaExec->timeout = (DEFAULT_TIMEOUT - (time()-$initRun));
                    if ($replicaExec->timeout < 1) {
                        $currentReplicaIndex = $index;
                        trigger_error("Stopping replica execution due to max run time. Next active table ".$replica['source'], E_USER_NOTICE);
                        break;
                    }
                    ++$processedTables;
                    try {
                        trigger_error("*****Starting ".($replica['reverse']?"outbound":"inbound")." replica for table ".$replica['source'], E_USER_NOTICE);
                        try {
                            $replicaExec = $replicationManager->runReplica($connection, $replicaExec);
                        } catch (RESTException $e){
                            if ($e->getCode() == 404) {
                                trigger_error("dsConfigId ".$replicaExec->dsConfig->id." not found in server for table ".$replica['source'].". Ignoring execution", E_USER_NOTICE);
                                --$processedTables;
                                continue;
                            } else {
                                throw $e;
                            }
                        }
                        $execution[EXECUTION_REPLICAS][$replica['ds']->id] = $replicaExec;
                        if ($replicaExec->timedout) {
                            $currentReplicaIndex = $index;
                            trigger_error("Stopping replica execution due to max run time, with partial replication of table ".$replica['source'], E_USER_NOTICE);
                            $stats= ' Working on replication of '.$replica['source'];
                            $execCode=EXEC_PAUSE_STATUS;
                            break;
                        }
                    } catch (Exception $e) {
                        trigger_error("Error found execution replica of table ".$replica['source']. ": ".$e->getMessage(), E_USER_NOTICE);   
                        $failedTables[$replica['source']] = $e->getMessage();
                    }
                }
            }
        } finally {
            if ($execCode !== EXEC_PAUSE_STATUS) {
                $execCode = count($failedTables) == 0 ? EXEC_OK_STATUS : EXEC_ERROR_STATUS;
            }
            if (empty($stats)){
                $stats = count($failedTables) == 0 ? " Finished successfully." : " ".count($failedTables)." Errors found. First: Table ".array_key_first($failedTables)." ".reset($failedTables);
            }
            $execution[EXECUTION_STATUS] = $execCode;
            $execution[EXECUTION_STATS] = $stats;
            $execution[EXECUTION_INDEX] = $index;
            if ($index == count($dsReplicaArray)) {
                $execution[EXECUTION_END_TS]=time();
            }
            update_option(CONFIG_PARAM_EXECUTION, $execution);
            $replicas[REPLICAS_LIST]=$dsReplicaArray;
            unset($replicationManager);
            unset($restClient);
            if (isset($connection)) mysqli_close($connection);
        }
    } else {
        trigger_error("Plugin is not connected to agent. Ignoring", E_USER_NOTICE);
    }
}

function dsmirror_num_active($dsReplicaArray, $reverse = false)
{
    $res = 0;
    foreach ($dsReplicaArray as $dsReplicaConfig) {
        if ($dsReplicaConfig['found'] && $dsReplicaConfig['active'] && $dsReplicaConfig['reverse'] === $reverse) {
            ++$res;
        }
    }
    return $res;
}

