<?php
/**
 * Author: Tecknolab
 * Author URI: https://tecknolab.com/
 * License: GPLv2
 **/
require_once 'dsmirror-plugin-common.php';


function dsmirror_properties($registration, $restClientSetup, $replicationManager=NULL, $force=false)
{
    $res=NULL;
    if (isset($registration[REGISTRATION_ACTIVE]) && $registration[REGISTRATION_ACTIVE]) {
        if (!isset($registration[REGISTRATION_PROPERTIES]) || $force || (time() - $registration[REGISTRATION_PROPERTIES_TS]) > MAX_PROPERTIES_INTERVAL) {
            try {
                $appId = $registration[REGISTRATION_APPID];
                $appSecret = $registration[REGISTRATION_SECRET];
                $client=isset($restClientSetup)&&!isset($replicationManager) ? $restClientSetup(DSMIRROR_ENDPOINT,$appId,$appSecret) : NULL;
                $manager=$replicationManager;
                if (!isset($manager)) {
                    $client = $restClientSetup($appId,$appSecret);
                    $manager = new dsmirror\ReplicationManager(dsmirror\ReplicationServiceClient::createCustom($client));
                }
                $registration[REGISTRATION_PROPERTIES] = $manager->serverProps();
                $registration[REGISTRATION_PROPERTIES_TS] = time();
                update_option(CONFIG_PARAM_REGISTRATION,$registration);
            } catch (\Exception $e) {
                trigger_error("Error retrieving DSmirror agent properties: ".$e->getMessage(), E_USER_NOTICE);
            } finally {
                if (!isset($replicationManager)) {
                    unset($manager);
                    if (isset($client)) {unset($client);}
                }
            }
        }
        if (isset($registration[REGISTRATION_PROPERTIES])) {
            $res = $registration[REGISTRATION_PROPERTIES];
        }
    }
    return $res;
}

function dsmirror_home($restClientSetup, $form_url='tools.php?page=dsmirror-plugin'){
    $registration = get_option(CONFIG_PARAM_REGISTRATION, Array(REGISTRATION_ACTIVE=>false, REGISTRATION_STATUS=>'Not registered!', REGISTRATION_REVERSE=>false,  REGISTRATION_APPID=>NULL, REGISTRATION_SECRET=>NULL));
    $registered = $registration[REGISTRATION_ACTIVE];
    $execution = get_option(CONFIG_PARAM_EXECUTION, Array(EXECUTION_STATUS=>EXEC_NEVER_STATUS, EXECUTION_STATS=>'',EXECUTION_INDEX=>0,EXECUTION_START_TS=>NULL,EXECUTION_END_TS=>NULL,EXECUTION_REPLICAS=>Array()) );
    $replicas = get_option(CONFIG_PARAM_REPLICAS, array(REPLICAS_LIST=>Array(),REPLICAS_CREATE_TABLE=>true,REPLICAS_SCHEDULING=>SCHED_HOURLY));
    
    $execCode = $execution[EXECUTION_STATUS];
    $execStatus = EXEC_STR_ARRAY[$execCode];
    if (isset($execution[EXECUTION_START_TS])) {
        $lastStart = time()-$execution[EXECUTION_START_TS];
        if ($lastStart >= 0 && $lastStart < 60) {
            $execStatus = $execStatus . ' Last execution started '.$lastStart.' sec ago.';
        } else if ($lastStart >= 60 && $lastStart < 3600) {
            $execStatus = $execStatus . ' Last execution started '.(int)($lastStart/60).' min ago.';
        } else if ($lastStart >= 3600 && $lastStart < 864000) {
            $execStatus = $execStatus . ' Last execution started '.(int)($lastStart/3600).' hours ago.';
        }
        if (isset($execution[EXECUTION_END_TS])) {
            $duration = $execution[EXECUTION_END_TS] - $execution[EXECUTION_START_TS];
            if ($duration > 0 && $duration < 120) {
                $execStatus = $execStatus . ' Duration: '.$duration.' sec.';
            } else if ($duration >= 120 && $duration < 7200) {
                $execStatus = $execStatus . ' Duration: '.(int)($duration/60).' min.';
            } else if ($duration >= 7200 && $duration < 64000) {
                $execStatus = $execStatus . ' Duration: '.(int)($duration/3660).' hours.';
            }
        }
    }
    $execStatus = $execStatus . $execution[EXECUTION_STATS];
    // Check if WP_cron is working properly...
    $nextCron = (dsmirror_next_scheduled() +  60 - time());
    if ($nextCron < 0) $execStatus = $execStatus. ' Invalid background launch, please check '.dsmirror_schedule_engine().' is running';
    $dsReplicaArray = $replicas[REPLICAS_LIST];
    trigger_error("Initial replica config ".json_encode($dsReplicaArray), E_USER_NOTICE);
    $regStatus = $registration[REGISTRATION_STATUS];
    $regReverse = $registration[REGISTRATION_REVERSE];
    $appId = $registration[REGISTRATION_APPID];
    $appSecret = $registration[REGISTRATION_SECRET];
    $createTable = $replicas[REPLICAS_CREATE_TABLE];
    $sched = $replicas[REPLICAS_SCHEDULING];
    $testResults='';
    $forcePropertiesReload=false;
    
    // Check whether the button has been pressed AND also check the nonce
    if (isset($_POST['test_button']) && check_admin_referer('test_button_clicked', '__test_nonce')) {
        if (isset($_POST['test_table'])) {
            $index = dsmirror_find_replica_in_config($dsReplicaArray, sanitize_text_field($_POST['test_table']));
            if ($index >= 0) {
                $replicaExec = new dsmirror\ReplicaExec(NULL);
                $replica = new dsmirror\DSReplica(NULL);
                $options = new dsmirror\DSReplicaOptions(NULL);
                $options->createTable = $createTable ? true : false;
                $options->targetTableName = $dsReplicaArray[$index]['target'];
                $replica->options = $options;
                $replica->dsConfigId = $dsReplicaArray[$index]['ds']->id;
                $replicaExec->dsReplica = $replica;
                $replicaExec->dsConfig = $dsReplicaArray[$index]['ds'];
                $testResults=dsmirror_test_replica($replicaExec,$appId, $appSecret, $restClientSetup);
            } else {
                $testResults="Error looking up replica configuration. DS config id not found: ".$_POST['test_table'];
            }
        }
    }
    if (isset($_POST['register_button']) && check_admin_referer('register_button_clicked', '__register_nonce')) {
        // the button has been pressed AND we've passed the security check
        if (!empty($_POST['dsmirror_app_id']) && !empty($_POST['dsmirror_app_secret']))
        {
            $appId = sanitize_text_field( $_POST['dsmirror_app_id']);
            $appSecret = sanitize_text_field( $_POST['dsmirror_app_secret']);
            try {
                $restClient = $restClientSetup($appId,$appSecret);
                $replicationManager = new dsmirror\ReplicationManager(dsmirror\ReplicationServiceClient::createCustom($restClient));
                $availableDSConfigArray = $replicationManager->listAvailableDSConfig();
                $regStatus = "Registered OK!";
                $registered = true;
                $dsReplicaArray=dsmirror_process_replica($dsReplicaArray, $availableDSConfigArray);
                if (isset($_POST['reverse_register'])){
                    $regReverse=true;
                    try {
                        $connection = mysqli_connect(DB_HOST, DB_USER, DB_PASSWORD);
                        mysqli_select_db($connection, DB_NAME);
                        $availableDSConfigReverseArray = $replicationManager->registerReverseDSConfig(DB_NAME, $connection);
                        trigger_error("Registered reverse dsConfig: ".json_encode($availableDSConfigReverseArray), E_USER_NOTICE);
                        $dsReplicaArray=dsmirror_process_replica($dsReplicaArray, array_merge($availableDSConfigArray, $availableDSConfigReverseArray));
                    } finally {
                        if (isset($connection))
                            mysqli_close($connection);
                    }
                } else {
                    $regReverse=false;
                }
                // Force reload of server properties
                $forcePropertiesReload=true;
            } catch (Exception $e) {
                $regStatus = "Not registered. ".$e->getMessage();
                $registered = false;
            } finally {
                unset($replicationManager);
                unset($restClient);
            }            
            $registration[REGISTRATION_ACTIVE]=$registered;
            $registration[REGISTRATION_STATUS]=$regStatus;
            $registration[REGISTRATION_APPID]=$appId;
            $registration[REGISTRATION_SECRET]=$appSecret;
            $registration[REGISTRATION_REVERSE]=$regReverse;
            update_option(CONFIG_PARAM_REGISTRATION,$registration);
            $replicas[REPLICAS_LIST] = $dsReplicaArray;
            update_option(CONFIG_PARAM_REPLICAS,$replicas);
        }
        else 
        {
            $regStatus = "Please fill up application Id and secret!";
        }
    }
    $properties=dsmirror_properties($registration, $restClientSetup, NULL, $forcePropertiesReload);
    
    if ($registered) {
        trigger_error("Current properties: ".json_encode($properties), E_USER_NOTICE);
        if (isset($properties->LICENSE_MAX_TABLES)) {
            $maxTables = $properties->LICENSE_MAX_TABLES;
        }
        if (isset($properties->LICENSE_MAX_ROWS)) {
            $maxRows = $properties->LICENSE_MAX_ROWS;
        }
        $limits="";
        if (isset($maxTables)) $limits = " License limited to ".$maxTables. " tables";
        if (isset($maxRows)) $limits = (empty($limits) ? " License limited to ".$maxRows." replicated rows per table" : $limits." and ".$maxRows." replicated rows per table");        
        trigger_error("Current limits: ".$limits, E_USER_NOTICE);
    } 
    if (isset($_POST['update_button']) && check_admin_referer('update_button_clicked', '__update_nonce')) {
        $updateError=dsmirror_validate_update_form($dsReplicaArray, $maxTables);
        if (empty($updateError)) {
            $index=0;
            $createTable=isset($_POST['create_table']);
            $sched=isset($_POST['schedule'])&&array_key_exists($_POST['schedule'], SCHED_DESC_ARRAY)?$_POST['schedule']:$sched;
            foreach ($dsReplicaArray as $dsReplicaConfig)
            {
                $dsReplicaArray[$index]=dsmirror_update_replica_config($dsReplicaConfig);
                $index++;
            }
            trigger_error("Updated replica config Array ".json_encode($dsReplicaArray), E_USER_NOTICE);
            $replicas[REPLICAS_LIST]=$dsReplicaArray;
            $replicas[REPLICAS_CREATE_TABLE]=$createTable;
            $replicas[REPLICAS_SCHEDULING]=$sched;
            update_option(CONFIG_PARAM_REPLICAS,$replicas);
        } else {
            trigger_error("Error found in update validation: ".$updateError, E_USER_NOTICE);
        }
    }
    
    $isAnyReplicaActive = dsmirror_is_any_active($dsReplicaArray);
    
    // Start building the page    
    echo '<div class="wrap">';
    echo '<div style="display: flex; align-items: left;justify-content: left"><div><img src="'.esc_url(DSMIRROR_LOGO_URL).'" alt="DSmirror logo" width="80" height="80" ></div>';
    echo '<div><h2 style="padding: 10px">DSmirror settings. Data Replication for WordPress</h2></div></div>';
    if (!$registered) {
       // Provide more detailed setup instructions             
        echo 'DSmirror is a solution for easy WP database tables replication and sync from/to an external database.<br>';
        echo 'It uses an agent with direct access to your external DB, but it does not require direct connectivity with your WP database<br>';
        echo 'Go to <a href="'.esc_url(DSMIRROR_URL).'" target="_blank">DSmirror site</a> and request a <a href="'.esc_url(DSMIRROR_FREE_LICENSE_URL).'" target="_blank">free license</a>.<br>';
        echo 'You will receive by email a combination of application id and secret.<br><br>';
        echo 'Launch a DSmirror agent in a computer with access to your external database ';
        echo '(check detailed instructions <a href="'.esc_url(DSMIRROR_INSTALL_URL).'" target="_blank">here</a>).<br>';
        echo 'Once running and registered, use the app id and secret to connect your WP plugin below<br>';
    }
    
    echo '<h3>DSmirror application registration</h3>';
    echo '<form action="'.esc_url($form_url).'" method="post">';
    
    // this is a WordPress security feature - see: https://codex.wordpress.org/WordPress_Nonces
    wp_nonce_field('register_button_clicked', '__register_nonce');
    echo '<div style="display:grid;grid-template-columns: max-content max-content;grid-gap:5px;">';
    echo '<label for="dsmirror_app_id">Application id: </label>';
    echo '<input type="text" id="dsmirror_app_id" name="dsmirror_app_id" value="'.(!empty($appId)?esc_attr($appId):'').'" size=50>';
    echo '<label for="dsmirror_app_secret">Aplication secret: </label>';
    echo '<input type="password" id="dsmirror_app_secret" name="dsmirror_app_secret" value="'.(!empty($appSecret)?esc_attr($appSecret):'').'" size=50>';
    echo '</div>';
    $regReverseValue=!$registered ? isset($_POST['reverse_register']): $regReverse;
    echo '<label for="reverse_register"><i>Enable replication in outbound direction (from WP to server)</i> </label>';
    echo '<input type="checkbox" name="reverse_register" '.($regReverseValue?'checked':'').'/>';
    echo '<input type="hidden" value="true" name="register_button" />';
    submit_button('Register/Refresh','primary','register');
    echo '<div style="display:grid;grid-template-columns: max-content max-content max-content; grid-gap:5px;">';
    echo '<label for="register_status">Registration status: </label>';
    echo '<input style="background-color:'.($registered ? 'PaleGreen':'LemonChiffon').';" type="text" readonly name="register_status" value="'.esc_attr($regStatus.(isset($limits)?$limits:'')).'" size=50>';
    echo (!empty($limits)?'<a href="'.esc_url(DSMIRROR_BUY_URL).'">Buy an unrestricted license now!</a>':'<a href="#"></a>');
    echo '<label for="exec_status">Execution status: </label>';
    echo '<input style="background-color:'.esc_attr(EXEC_STATUS_COLOR_ARRAY[$execCode]).';" type="text" readonly name="exec_status" value="'.esc_attr($execStatus).'" size=80>';
    if (strpos($execStatus, dsmirror_schedule_engine())) echo  '<a href="'.esc_url(dsmirror_schedule_engine_info()).'" target="_blank">Find info on '.dsmirror_schedule_engine().' issues</a>';
    echo '</div>';
    echo '</form>';    
    echo '</div>';
    if ($registered) {
        echo '<div class="wrap">';
        $sortedArray = Array();
        $sortedReverseArray = Array();
        foreach ($dsReplicaArray as $dsReplicaConfig) {
            if ($dsReplicaConfig['found']) {
                if (!$dsReplicaConfig['reverse']) {
                    $sortedArray[$dsReplicaConfig['source']]=$dsReplicaConfig;
                } else if ($regReverse){
                    $sortedReverseArray[$dsReplicaConfig['source']]=$dsReplicaConfig;
                }
            }
        }
        ksort($sortedArray, SORT_NATURAL);
        ksort($sortedReverseArray, SORT_NATURAL);
        if (empty($sortedArray) && empty($sortedReverseArray)) {
            echo '<h4>No tables available for replication in DSmirror agent. Please configure your external data source in DSmirror agent</h4>';
        } else {
            echo '<h3> Data Replication Configuration</h3>';
            echo '<form action="'.esc_url($form_url).'" method="post">';
            wp_nonce_field('update_button_clicked', '__update_nonce');
            echo '<label for="create_table"><i>Automatically create replicated tables in target database if not exists</i> </label>';
            echo '<input type="checkbox" name="create_table" '.($createTable?'checked':'').'/><br>';
            echo '<label for="schedule">Frequency for replication: </label>';
            echo '<select name="schedule" id="schedule">';
            foreach (SCHED_DESC_ARRAY as $schedValue => $schedDesc) {
                echo '<option value="'.esc_attr($schedValue).'" '.($sched==$schedValue?'selected ':' ').(!isset($properties->LICENSE_GRANTS)&&SCHED_SECONDS_ARRAY[$schedValue]>0 &&SCHED_SECONDS_ARRAY[$schedValue]<3600?'disabled':'').'>'.esc_attr($schedDesc).'</option>';
            }
            echo '</select>';
            echo '<br>Select the tables to replicate and optionally adapt its name in target database';
        }
        $formItems = Array();
        if (!empty($updateError)) {
            echo '<br><textarea style="background-color:Tomato;" id="update_error" name="update_error" readonly rows="1" cols="80">'.esc_textarea($updateError).'</textarea><br>';
        }
        echo dsmirror_jquery_script();
        echo '<script type="text/javascript">';
        echo '(function( $ ) {"use strict";$(document).ready(function() {$(\'#checkbox-all\').change(function () {$(\'[id^="cb-inbound-"]\').prop(\'checked\', $(this).prop(\'checked\'));});$(\'#checkbox-rev-all\').change(function () {$(\'[id^="cb-outbound-"]\').prop(\'checked\', $(this).prop(\'checked\'));});});})(jQuery);';
        echo '</script>';
        if (!empty($sortedArray)){
            echo '<h4>Available tables for inbound replication (ingest into WP database):</h4>';
            echo '<div style="width: max-content; height: 120px; overflow-y: scroll;">';
            echo '<table style="position: relative;">';
            echo '<thead><tr><th style="top: 0;position: sticky;background: DimGrey;color: white;">Remote table</th><th style="top: 0;position: sticky;background: DimGrey;color: white;">WP replica</th><th style="top: 0;position: sticky;"><input type="checkbox" name="checkbox-all" id="checkbox-all"  /></th></tr></thead><tbody>';
            foreach ($sortedArray as $table => $dsReplicaConfig) {
                trigger_error("Display inbound replication config: ".json_encode($dsReplicaConfig), E_USER_NOTICE);
                $targetValue=(!empty($updateError) && isset($_POST['target-'.$dsReplicaConfig['ds']->id])) ? sanitize_text_field($_POST['target-'.$dsReplicaConfig['ds']->id]): $dsReplicaConfig['target'];
                $active=(!empty($updateError) ? isset($_POST['active-'.$dsReplicaConfig['ds']->id]) : $dsReplicaConfig['active']);
                echo '<tr><td style="min-width:150px;padding-left: 0.5rem;"><b>'.esc_attr($dsReplicaConfig['source']).'</b></td><td><input type="text" name="target-'.esc_attr($dsReplicaConfig['ds']->id).'" id="target-'.esc_attr($dsReplicaConfig['ds']->id).'" value="'.esc_attr($targetValue).'"/></td><td><input type="checkbox" name="active-'.$dsReplicaConfig['ds']->id.'" id="cb-inbound-'.esc_attr($dsReplicaConfig['ds']->id).'" '.($active?'checked':'').'/></td></tr>';
            }
            echo '</tbody></table>';
            echo '</div>';
        }
        if (!empty($sortedReverseArray)){
            echo '<h4>Available tables for outbound replication (ingest into remote database):</h4>';
            echo '<div style="width: max-content; height: 120px; overflow-y: scroll;">';
            echo '<table style="position: relative;">';
            echo '<thead><tr><th style="top: 0;position: sticky;background: DimGrey;color: white;">Remote replica</th><th style="top: 0;position: sticky;background: DimGrey;color: white;">WP table</th><th style="top: 0;position: sticky;"><input type="checkbox" name="checkbox-rev-all" id="checkbox-rev-all"  /></th></tr></thead><tbody>';
            foreach ($sortedReverseArray as $table => $dsReplicaConfig) {
                trigger_error("Display outbound replication config: ".json_encode($dsReplicaConfig), E_USER_NOTICE);
                $targetValue=(!empty($updateError) && isset($_POST['target-'.$dsReplicaConfig['ds']->id])) ? sanitize_text_field($_POST['target-'.$dsReplicaConfig['ds']->id]): $dsReplicaConfig['target'];
                $active=(!empty($updateError) ? isset($_POST['active-'.$dsReplicaConfig['ds']->id]) : $dsReplicaConfig['active']);
                echo '<tr><td><input type="text" name="target-'.esc_attr($dsReplicaConfig['ds']->id).'" id="target-'.esc_attr($dsReplicaConfig['ds']->id).'" value="'.esc_attr($targetValue).'"/></td><td style="min-width:150px;padding-left: 0.5rem;"><b>'.esc_attr($dsReplicaConfig['source']).'</b></td><td><input type="checkbox" name="active-'.$dsReplicaConfig['ds']->id.'" id="cb-outbound-'.esc_attr($dsReplicaConfig['ds']->id).'" '.($active?'checked':'').'/></td></tr>';
            }
            echo '</tbody></table>';
            echo '</div>';
        }
        if (!empty($sortedArray) || !empty($sortedReverseArray)) {
            echo '<input type="hidden" value="true" name="update_button" />';
            submit_button('Update','primary','update');
            echo '</form>';
        }
        echo '</div>';
        if ($isAnyReplicaActive) {
            echo '<div class="wrap">';
            echo '<h3>Test replication:</h3>';
            if (!empty($testResults)) {
                echo '<textarea id="test_results" name="test_results" readonly rows="2" cols="80">'.esc_textarea($testResults).'</textarea><br>';
            }
            echo 'For testing a table replica, select and press "Test". It will execute the process for a maximum of 10 rows to test it is OK<br><br>';
            echo '<form action="'.esc_url($form_url).'" method="post">';
            wp_nonce_field('test_button_clicked', '__test_nonce');
            echo '<label for="test_table">Select a replica:</label>';
            echo '<select name="test_table" id="test_table">';
            $activeTables=dsmirror_active_tables($dsReplicaArray);
            foreach ($activeTables as $dsConfigId => $replicaDesc) {
                echo '<option value="'.esc_attr($dsConfigId).'">'.esc_attr($replicaDesc).'</option>';
            }
            $activeTables=dsmirror_active_tables($dsReplicaArray,true);
            foreach ($activeTables as $dsConfigId => $replicaDesc) {
                echo '<option value="'.esc_attr($dsConfigId).'">'.esc_attr($replicaDesc).'</option>';
            }
            echo '</select>';
            echo '<input type="hidden" value="true" name="test_button" />';
            submit_button('Test','primary','test');
            echo '</form>';
        }
    }
}

function dsmirror_test_replica($replicaExec, $appId, $appSecret, $restClientSetup) {
    try {
        // Force the creation of a specific test replica
        $replicaExec->dsReplica->options->test = true;
        $connection = mysqli_connect(DB_HOST, DB_USER, DB_PASSWORD);
        mysqli_select_db($connection, DB_NAME);
        trigger_error("Connected to DB. Starting replica test for dsConfig ".$replicaExec->dsReplica->dsConfigId, E_USER_NOTICE);
        $restClient = $restClientSetup($appId,$appSecret);
        $replicationManager = new dsmirror\ReplicationManager(dsmirror\ReplicationServiceClient::createCustom($restClient));

        $replicaExec->timeout=DEFAULT_TIMEOUT;
        $replicaExec = $replicationManager->runReplica($connection, $replicaExec);
        if (!$replicaExec->timedout) {
            return "test replication executed OK for target table ".$replicaExec->dsReplica->options->targetTableName;
        } else {
            return "test replication timedout after ".DEFAULT_TIMEOUT." for target table ".$replicaExec->dsReplica->options->targetTableName;
        }
    } catch (Exception $e) {
        return "failed execution of replica for target table " .$replicaExec->dsReplica->options->targetTableName."\n".$e->getMessage();
    }
    finally {
        if (isset($replicationManager) && isset($replicaExec) && isset($replicaExec->dsReplica)) {
            try {$replicationManager->deleteReplica($replicaExec->dsReplica);} catch (Exception $ex){}
        }
        unset($replicationManager);
        unset($restClient);
        if (isset($connection))
           mysqli_close($connection);
    }
}

function dsmirror_process_replica($dsReplicaArray, $availableDSConfigArray)
{
    $index=0;
    foreach ($dsReplicaArray as $dsReplicaConfig) {
        $dsReplicaConfig['found']=false;
        $dsReplicaArray[$index]=$dsReplicaConfig;
        $index++;
    }
    foreach ($availableDSConfigArray as $dsConfig) {
        $index = dsmirror_find_replica_in_config($dsReplicaArray, $dsConfig->id);
        if ($index>=0) {
            $dsReplicaArray[$index]['found']=true;
        } else {
            // Create a new placeholder replica
            trigger_error("Processing new configured replica for dsConfig ".$dsConfig->id, E_USER_NOTICE);
            $dsReplicaConfig = array();
            $dsReplicaConfig['found'] = true;            
            $dsReplicaConfig['ds'] = $dsConfig;
            $dsReplicaConfig['active'] = false;
            $dsReplicaConfig['target'] = $dsConfig->tableName;
            $dsReplicaConfig['source'] = $dsConfig->tableName;
            $dsReplicaConfig['reverse'] = $dsConfig->reverse;
            $dsReplicaArray[]=$dsReplicaConfig;
        }
    }
    trigger_error("Processed replica config Array #".count($dsReplicaArray).": ".json_encode($dsReplicaArray), E_USER_NOTICE);
    
    return $dsReplicaArray;
}

function dsmirror_update_replica_config($dsReplicaConfig)
{
    trigger_error('Updating from form post the configuration of dsReplicaConfig: '.json_encode($dsReplicaConfig), E_USER_NOTICE);
    if (!empty($_POST['target-'.$dsReplicaConfig['ds']->id])) {
        $dsReplicaConfig['target']=sanitize_text_field($_POST['target-'.$dsReplicaConfig['ds']->id]);
    }
    $dsReplicaConfig['active']= isset($_POST['active-'.$dsReplicaConfig['ds']->id]);
    return $dsReplicaConfig;
}

function dsmirror_find_replica_in_config($dsReplicaArray, $dsConfigId)
{
    for ($index=0; $index<count($dsReplicaArray); ++$index) {
        if (!empty($dsReplicaArray[$index]['ds']) && $dsReplicaArray[$index]['ds']->id == $dsConfigId)
        {
            return $index;
        }
    }
    return -1;
}

function dsmirror_is_any_active($dsReplicaArray)
{
    foreach ($dsReplicaArray as $dsReplicaConfig) {
        if ($dsReplicaConfig['found'] && $dsReplicaConfig['active']) {
            return true;
        }
    }
    return false;
}

function dsmirror_active_tables($dsReplicaArray, $reverse = false)
{
    $res=array();
    foreach ($dsReplicaArray as $dsReplicaConfig) {
        if ($dsReplicaConfig['found'] && $dsReplicaConfig['active'] && $dsReplicaConfig['reverse'] === $reverse) {
            if (!$reverse)
                $res[$dsReplicaConfig['ds']->id]=$dsReplicaConfig['source'].'=>'.$dsReplicaConfig['target'];
            else
                $res[$dsReplicaConfig['ds']->id]=$dsReplicaConfig['target'].'<='.$res[$dsReplicaConfig['ds']->id]=$dsReplicaConfig['source'];
        }
    }
    asort($res, SORT_NATURAL);
    return $res;
}

function dsmirror_validate_update_form($dsReplicaArray, $maxTables)
{
    $res = "";
    // Regular replicas must have different targets from each other
    $tables=Array();
    $reverseTables=Array();
    $replicas=0;
    foreach ($dsReplicaArray as $dsReplicaConfig) {
        if ($dsReplicaConfig['found'] && isset($_POST['active-'.$dsReplicaConfig['ds']->id])) {
            $target = sanitize_text_field($_POST['target-'.$dsReplicaConfig['ds']->id]);
            ++$replicas;
            if (isset($maxTables) && $replicas > $maxTables) {
                $res = "Maximum number of tables to replicate with current license is  ".$maxTables;
                break;
            }
            $targetTables = ($dsReplicaConfig['reverse']) ? $reverseTables : $tables;
            if (in_array($target, $targetTables)) {
                $res = "Target table name ".$target." is duplicated in (".($dsReplicaConfig['reverse']?"outbound":"inbound").") replication configuration";
                break;
            } else {
                if ($dsReplicaConfig['reverse']) {
                    $reverseTables[]=$target;
                } else {
                    $tables[]=$target;
                }
            }
        }
    }
    return $res;
}